@@ -13,14 +13,13 @@ class SwarmTest(BaseAPIIntegrationTest):
         self._unlock_key = None
 
     def tearDown(self):
-        super(SwarmTest, self).tearDown()
         try:
             if self._unlock_key:
                 self.client.unlock_swarm(self._unlock_key)
         except docker.errors.APIError:
             pass
-
         force_leave_swarm(self.client)
+        super(SwarmTest, self).tearDown()
 
     @requires_api_version('1.24')
     def test_init_swarm_simple(self):
@@ -36,6 +35,35 @@ class SwarmTest(BaseAPIIntegrationTest):
         version_2 = self.client.inspect_swarm()['Version']['Index']
         assert version_2 != version_1
 
+    @requires_api_version('1.39')
+    def test_init_swarm_custom_addr_pool_defaults(self):
+        assert self.init_swarm()
+        results = self.client.inspect_swarm()
+        assert set(results['DefaultAddrPool']) == {'10.0.0.0/8'}
+        assert results['SubnetSize'] == 24
+
+    @requires_api_version('1.39')
+    def test_init_swarm_custom_addr_pool_only_pool(self):
+        assert self.init_swarm(default_addr_pool=['2.0.0.0/16'])
+        results = self.client.inspect_swarm()
+        assert set(results['DefaultAddrPool']) == {'2.0.0.0/16'}
+        assert results['SubnetSize'] == 24
+
+    @requires_api_version('1.39')
+    def test_init_swarm_custom_addr_pool_only_subnet_size(self):
+        assert self.init_swarm(subnet_size=26)
+        results = self.client.inspect_swarm()
+        assert set(results['DefaultAddrPool']) == {'10.0.0.0/8'}
+        assert results['SubnetSize'] == 26
+
+    @requires_api_version('1.39')
+    def test_init_swarm_custom_addr_pool_both_args(self):
+        assert self.init_swarm(default_addr_pool=['2.0.0.0/16', '3.0.0.0/16'],
+                               subnet_size=28)
+        results = self.client.inspect_swarm()
+        assert set(results['DefaultAddrPool']) == {'2.0.0.0/16', '3.0.0.0/16'}
+        assert results['SubnetSize'] == 28
+
     @requires_api_version('1.24')
     def test_init_already_in_cluster(self):
         assert self.init_swarm()
@@ -158,12 +186,14 @@ class SwarmTest(BaseAPIIntegrationTest):
 
     @requires_api_version('1.24')
     def test_inspect_node(self):
-        assert self.init_swarm()
+        node_id = self.init_swarm()
+        assert node_id
         nodes_list = self.client.nodes()
         assert len(nodes_list) == 1
         node = nodes_list[0]
         node_data = self.client.inspect_node(node['ID'])
         assert node['ID'] == node_data['ID']
+        assert node_id == node['ID']
         assert node['Version'] == node_data['Version']
 
     @requires_api_version('1.24')
@@ -205,3 +235,21 @@ class SwarmTest(BaseAPIIntegrationTest):
             self.client.remove_node(node_id, True)
 
         assert e.value.response.status_code >= 400
+
+    @requires_api_version('1.25')
+    def test_rotate_manager_unlock_key(self):
+        spec = self.client.create_swarm_spec(autolock_managers=True)
+        assert self.init_swarm(swarm_spec=spec)
+        swarm_info = self.client.inspect_swarm()
+        key_1 = self.client.get_unlock_key()
+        assert self.client.update_swarm(
+            version=swarm_info['Version']['Index'],
+            rotate_manager_unlock_key=True
+        )
+        key_2 = self.client.get_unlock_key()
+        assert key_1['UnlockKey'] != key_2['UnlockKey']
+
+    @requires_api_version('1.30')
+    @pytest.mark.xfail(reason='Can fail if eth0 has multiple IP addresses')
+    def test_init_swarm_data_path_addr(self):
+        assert self.init_swarm(data_path_addr='eth0')
