@@ -4,7 +4,8 @@ import tempfile
 import docker
 import pytest
 
-from .base import BaseIntegrationTest, BUSYBOX, TEST_API_VERSION
+from .base import BaseIntegrationTest, TEST_IMG, TEST_API_VERSION
+from ..helpers import random_name
 
 
 class ImageCollectionTest(BaseIntegrationTest):
@@ -71,8 +72,8 @@ class ImageCollectionTest(BaseIntegrationTest):
 
     def test_pull_with_tag(self):
         client = docker.from_env(version=TEST_API_VERSION)
-        image = client.images.pull('alpine', tag='3.3')
-        assert 'alpine:3.3' in image.attrs['RepoTags']
+        image = client.images.pull('alpine', tag='3.10')
+        assert 'alpine:3.10' in image.attrs['RepoTags']
 
     def test_pull_with_sha(self):
         image_ref = (
@@ -96,7 +97,7 @@ class ImageCollectionTest(BaseIntegrationTest):
 
     def test_save_and_load(self):
         client = docker.from_env(version=TEST_API_VERSION)
-        image = client.images.get(BUSYBOX)
+        image = client.images.get(TEST_IMG)
         with tempfile.TemporaryFile() as f:
             stream = image.save()
             for chunk in stream:
@@ -108,6 +109,32 @@ class ImageCollectionTest(BaseIntegrationTest):
         assert len(result) == 1
         assert result[0].id == image.id
 
+    def test_save_and_load_repo_name(self):
+        client = docker.from_env(version=TEST_API_VERSION)
+        image = client.images.get(TEST_IMG)
+        additional_tag = random_name()
+        image.tag(additional_tag)
+        self.tmp_imgs.append(additional_tag)
+        image.reload()
+        with tempfile.TemporaryFile() as f:
+            stream = image.save(named='{}:latest'.format(additional_tag))
+            for chunk in stream:
+                f.write(chunk)
+
+            f.seek(0)
+            client.images.remove(additional_tag, force=True)
+            result = client.images.load(f.read())
+
+        assert len(result) == 1
+        assert result[0].id == image.id
+        assert '{}:latest'.format(additional_tag) in result[0].tags
+
+    def test_save_name_error(self):
+        client = docker.from_env(version=TEST_API_VERSION)
+        image = client.images.get(TEST_IMG)
+        with pytest.raises(docker.errors.InvalidArgument):
+            image.save(named='sakuya/izayoi')
+
 
 class ImageTest(BaseIntegrationTest):
 
