@@ -15,7 +15,7 @@ class NodesTest(unittest.TestCase):
 
     def test_list_get_update(self):
         client = docker.from_env(version=TEST_API_VERSION)
-        client.swarm.init('eth0', listen_addr=helpers.swarm_listen_addr())
+        client.swarm.init('127.0.0.1', listen_addr=helpers.swarm_listen_addr())
         nodes = client.nodes.list()
         assert len(nodes) == 1
         assert nodes[0].attrs['Spec']['Role'] == 'manager'
