@@ -11,6 +11,7 @@ import unittest
 
 
 from docker.api.client import APIClient
+from docker.constants import IS_WINDOWS_PLATFORM
 from docker.errors import DockerException
 from docker.utils import (
     convert_filters, convert_volume_binds, decode_json_header, kwargs_from_env,
@@ -83,15 +84,17 @@ class KwargsFromEnvTest(unittest.TestCase):
                           DOCKER_CERT_PATH=TEST_CERT_DIR,
                           DOCKER_TLS_VERIFY='1')
         kwargs = kwargs_from_env(assert_hostname=False)
-        assert 'https://192.168.59.103:2376' == kwargs['base_url']
+        assert 'tcp://192.168.59.103:2376' == kwargs['base_url']
         assert 'ca.pem' in kwargs['tls'].ca_cert
         assert 'cert.pem' in kwargs['tls'].cert[0]
         assert 'key.pem' in kwargs['tls'].cert[1]
         assert kwargs['tls'].assert_hostname is False
         assert kwargs['tls'].verify
+
+        parsed_host = parse_host(kwargs['base_url'], IS_WINDOWS_PLATFORM, True)
         try:
             client = APIClient(**kwargs)
-            assert kwargs['base_url'] == client.base_url
+            assert parsed_host == client.base_url
             assert kwargs['tls'].ca_cert == client.verify
             assert kwargs['tls'].cert == client.cert
         except TypeError as e:
@@ -102,15 +105,16 @@ class KwargsFromEnvTest(unittest.TestCase):
                           DOCKER_CERT_PATH=TEST_CERT_DIR,
                           DOCKER_TLS_VERIFY='')
         kwargs = kwargs_from_env(assert_hostname=True)
-        assert 'https://192.168.59.103:2376' == kwargs['base_url']
+        assert 'tcp://192.168.59.103:2376' == kwargs['base_url']
         assert 'ca.pem' in kwargs['tls'].ca_cert
         assert 'cert.pem' in kwargs['tls'].cert[0]
         assert 'key.pem' in kwargs['tls'].cert[1]
         assert kwargs['tls'].assert_hostname is True
         assert kwargs['tls'].verify is False
+        parsed_host = parse_host(kwargs['base_url'], IS_WINDOWS_PLATFORM, True)
         try:
             client = APIClient(**kwargs)
-            assert kwargs['base_url'] == client.base_url
+            assert parsed_host == client.base_url
             assert kwargs['tls'].cert == client.cert
             assert not kwargs['tls'].verify
         except TypeError as e:
@@ -272,6 +276,11 @@ class ParseHostTest(unittest.TestCase):
             'tcp://',
             'udp://127.0.0.1',
             'udp://127.0.0.1:2375',
+            'ssh://:22/path',
+            'tcp://netloc:3333/path?q=1',
+            'unix:///sock/path#fragment',
+            'https://netloc:3333/path;params',
+            'ssh://:clearpassword@host:22',
         ]
 
         valid_hosts = {
@@ -281,7 +290,7 @@ class ParseHostTest(unittest.TestCase):
             'http://:7777': 'http://127.0.0.1:7777',
             'https://kokia.jp:2375': 'https://kokia.jp:2375',
             'unix:///var/run/docker.sock': 'http+unix:///var/run/docker.sock',
-            'unix://': 'http+unix://var/run/docker.sock',
+            'unix://': 'http+unix:///var/run/docker.sock',
             '12.234.45.127:2375/docker/engine': (
                 'http://12.234.45.127:2375/docker/engine'
             ),
@@ -294,6 +303,9 @@ class ParseHostTest(unittest.TestCase):
             '[fd12::82d1]:2375/docker/engine': (
                 'http://[fd12::82d1]:2375/docker/engine'
             ),
+            'ssh://': 'ssh://127.0.0.1:22',
+            'ssh://user@localhost:22': 'ssh://user@localhost:22',
+            'ssh://user@remote': 'ssh://user@remote:22',
         }
 
         for host in invalid_hosts:
@@ -304,7 +316,7 @@ class ParseHostTest(unittest.TestCase):
             assert parse_host(host, None) == expected
 
     def test_parse_host_empty_value(self):
-        unix_socket = 'http+unix://var/run/docker.sock'
+        unix_socket = 'http+unix:///var/run/docker.sock'
         npipe = 'npipe:////./pipe/docker_engine'
 
         for val in [None, '']:
@@ -449,8 +461,8 @@ class UtilsTest(unittest.TestCase):
         tests = [
             ({'dangling': True}, '{"dangling": ["true"]}'),
             ({'dangling': "true"}, '{"dangling": ["true"]}'),
-            ({'exited': 0}, '{"exited": [0]}'),
-            ({'exited': [0, 1]}, '{"exited": [0, 1]}'),
+            ({'exited': 0}, '{"exited": ["0"]}'),
+            ({'exited': [0, 1]}, '{"exited": ["0", "1"]}'),
         ]
 
         for filters, expected in tests:
@@ -483,9 +495,12 @@ class PortsTest(unittest.TestCase):
         assert external_port == [("127.0.0.1", "1000")]
 
     def test_split_port_with_protocol(self):
-        internal_port, external_port = split_port("127.0.0.1:1000:2000/udp")
-        assert internal_port == ["2000/udp"]
-        assert external_port == [("127.0.0.1", "1000")]
+        for protocol in ['tcp', 'udp', 'sctp']:
+            internal_port, external_port = split_port(
+                "127.0.0.1:1000:2000/" + protocol
+            )
+            assert internal_port == ["2000/" + protocol]
+            assert external_port == [("127.0.0.1", "1000")]
 
     def test_split_port_with_host_ip_no_port(self):
         internal_port, external_port = split_port("127.0.0.1::2000")
@@ -538,6 +553,10 @@ class PortsTest(unittest.TestCase):
         with pytest.raises(ValueError):
             split_port("0.0.0.0:1000:2000:tcp")
 
+    def test_split_port_invalid_protocol(self):
+        with pytest.raises(ValueError):
+            split_port("0.0.0.0:1000:2000/ftp")
+
     def test_non_matching_length_port_ranges(self):
         with pytest.raises(ValueError):
             split_port("0.0.0.0:1000-1010:2000-2002/tcp")
